/**
 *  SimpleRenderer
 *
 *  Copyright (C) 2005  Yves Radunz, Volker Grabsch
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

import java.awt.*;
import java.awt.event.*;
import javax.media.opengl.*;
import com.sun.opengl.utils.Animator;

public abstract class SimpleRenderer implements GLEventListener,
						WindowListener,
						MouseListener,
						MouseMotionListener,
						KeyListener
{
    private final Frame frame;
    private final Animator animator;

    private double rotX = 0.0;
    private double rotY = 0.0;
    private int drawableWidth = 1;
    private int drawableHeight = 1;
    private int oldMouseX = 0;
    private int oldMouseY = 0;

    /********************************************************
     *
     *  Constructor
     *
     ********************************************************/

    public SimpleRenderer()
    {
	// OpenGL-Canvas
	GLCanvas canvas = new GLCanvas();

	// OpenGL-Animator
	animator = new Animator(canvas);

	// Frame
	frame = new Frame(title);
	frame.add(canvas);
	frame.setSize(400, 400);

	// register listeners
	canvas.addGLEventListener(this);
	canvas.addMouseListener(this);
	canvas.addMouseMotionListener(this);
	canvas.addKeyListener(this);
	frame.addWindowListener(this);
	frame.addKeyListener(this);

	// run!
	frame.show();
	start();
    }


    /********************************************************
     *
     *  Utilities
     *
     ********************************************************/

    public String title = "SimpleRenderer";
    public String[] initText = new String[]{};
    public String[] doneText = new String[]{};

    /**
     * Show status line
     */
    public void showLine(String line)
    {
	while (line.length() < 78)
	    line += " ";
	line += "\r";

    	System.out.print(line);
	System.out.flush();
    }

    /**
     * Create status line in a special format
     */
    public String createStatusLine(String[] items, int[] sizes)
    {
	String line = "  |";
	for (int i=0; i<items.length; i++)
	    {
		String item = items[i];
		while (item.length() < sizes[i])
		    item = " "+item;

		line += " "+item+" |";
	    }
	return line;
    }

    /**
     * Create header for a status line
     */
    public String createStatusHeader(String[] items, int[] sizes, char underline)
    {
	String line = createStatusLine(items, sizes);
	String header = "  ";

	boolean first = true;
	for (int i=0, len=line.length(); i<len; i++)
	    {
		char c = line.charAt(i);
		switch (c)
		    {
		    case '|':
			header += " ";
			first = false;
			break;

		    default:
			if (!first)
			    header += underline;
			break;
		    }
	    }
	return header;
    }

    /**
     * Show a bigger portion of text
     */
    public void showText(String[] text)
    {
	for (int i=0; i<text.length; i++)
	    System.out.println(text[i]);
	System.out.flush();
    }

    public void start()
    {
	animator.start();
    }

    public void stop()
    {
	animator.stop();
    }

    public void toggle()
    {
	if (animator.isAnimating())
	    animator.stop();
	else
	    animator.start();
    }

    public void quit()
    {
	stop();
	showText(new String[]{""});
	showText(doneText);
	System.exit(0);
    }


    /********************************************************
     *
     *  GLEventListener methods
     *
     ********************************************************/

    public void init(GLAutoDrawable drawable)
    {
	// initialize scene
	init();

	// show init text
	showText(initText);

	// initialize OpenGL
	GL gl = drawable.getGL();
	gl.setSwapInterval(1);
	gl.glEnable(GL.GL_LIGHTING);
	gl.glEnable(GL.GL_LIGHT0);
	gl.glEnable(GL.GL_DEPTH_TEST);
	gl.glEnable(GL.GL_SMOOTH);
    }

    public void reshape(GLAutoDrawable drawable,
			int x, int y, int width, int height)
    {
	drawableWidth = width;
	drawableHeight = height;
    }

    public void display(GLAutoDrawable drawable)
    {
	GL gl = drawable.getGL();
	gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);
	gl.glLoadIdentity();
	gl.glRotated(360*rotX, 0, 1, 0);
	gl.glRotated(360*rotY, 1, 0, 0);
	glRender(gl);
    }

    public void displayChanged(GLAutoDrawable drawable,
			       boolean modeChanged, boolean deviceChanged)
    {
    }


    /********************************************************
     *
     *  WindowListener methods
     *
     ********************************************************/

    public void windowActivated(WindowEvent e)
    {
    }

    public void windowClosed(WindowEvent e)
    {
    }

    public void windowClosing(WindowEvent e)
    {
	quit();
    }

    public void windowDeactivated(WindowEvent e)
    {
    }

    public void windowDeiconified(WindowEvent e)
    {
    }

    public void windowIconified(WindowEvent e)
    {
    }

    public void windowOpened(WindowEvent e)
    {
    }


    /********************************************************
     *
     *  Mouse(Motion)Listener methods
     *
     ********************************************************/

    public void mouseEntered(MouseEvent e)
    {
    }

    public void mouseExited(MouseEvent e)
    {
    }

    public void mousePressed(MouseEvent e)
    {
	oldMouseX = e.getX();
	oldMouseY = e.getY();
    }

    public void mouseReleased(MouseEvent e)
    {
    }
    
    public void mouseClicked(MouseEvent e)
    {
	if (e.getClickCount() == 2)
	    mouseDoubleClicked();
    }

    public void mouseDragged(MouseEvent e)
    {
	int x = e.getX();
	int y = e.getY();

	rotX += - (x-oldMouseX) / (double)drawableWidth;
	rotY += + (y-oldMouseY) / (double)drawableHeight;

	oldMouseX = x;
	oldMouseY = y;
    }

    public void mouseMoved(MouseEvent e)
    {
    }


    /********************************************************
     *
     *  KeyListener methods
     *
     ********************************************************/

    public void keyPressed(KeyEvent e)
    {
	keyTyped(Character.toUpperCase(e.getKeyChar()));
	keyFxTyped(e.getKeyCode()-(KeyEvent.VK_F1-1));

	if (e.getKeyCode() == KeyEvent.VK_ESCAPE)
	    quit();
    }

    public void keyReleased(KeyEvent e)
    {
    }

    public void keyTyped(KeyEvent e)
    {
    }


    /********************************************************
     *
     *  Methods to override
     *
     ********************************************************/

    /**
     * Initialize scene
     */
    public abstract void init();

    /**
     * Render scene
     */
    public abstract void glRender(GL gl);

    /**
     * React on double click
     */
    public void mouseDoubleClicked() {}

    /**
     * React on key
     */
    public void keyTyped(char key) {}

    /**
     * React on function (F1, ...) key
     */
    public void keyFxTyped(int fkey) {}
}
